#include <stdlib.h>
#include <math.h>
#include <tcl.h>
#include <tk.h>

#include "./factorcountInt.h"

/*-----------------------------------------------------
* int createThread (char *argv[])--
*    Create a new thread with a Tcl interpreter in it
*    Initialize the interp and start it running a
*    predefined script.
* Arguments:
*   argv	array of arguments from the command line
* 
* Results:
*   Returns TCL_OK or TCL_ERROR
* 
* Side Effects:
*   New thread, etc.
------------------------------------------------------*/
int createThread (char *argv[]) {
  Tcl_Interp *interp;     /* Interpreter for application. */
  int rtn;

  // Create the interp and initialize it.
  Tcl_FindExecutable(argv[0]);

  interp = Tcl_CreateInterp();

  if (Tcl_Init(interp) == TCL_ERROR) {
      printf("FAIL TCL_INIT\n");
      return TCL_ERROR;
  }

  if (Tk_Init(interp) == TCL_ERROR) {
      printf("FAIL TK_INIT\n");
      return TCL_ERROR;
  }
  
  // Add the factorcount command
  // factorcount_Cmd is defined in factor.c

  Tcl_CreateObjCommand(interp, "factorcount", 
      Factorcount_Cmd, (ClientData) NULL, NULL);

  // Run the Tcl script file - hardcoded for myScript.tcl

  rtn = Tcl_Eval(interp, "source myScript.tcl");

  if (rtn != TCL_OK) {
      printf("Failed Tcl_Eval: %d \n%s\n", rtn,
         Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY));
      exit(-1);
  }
}


main(int argc, char *argv[]) {
  // function variables
  int i;                  // Counter
  int threadCount;        // Number of threads to creats 
                          // read from command line
  Tcl_ThreadId *threadID;          // Returned by Tcl_CreateThread

  sscanf(argv[1], "%d", &threadCount);
  
  threadID = calloc(threadCount, sizeof(Tcl_ThreadId));
  
  for (i=0; i< threadCount; i++) {
    if (TCL_OK != Tcl_CreateThread(&threadID[i], 
        createThread, argv, TCL_THREAD_STACK_DEFAULT,  
	TCL_THREAD_JOINABLE)) {
	printf("Failed to create thread #%d\n", i);
	Tcl_Exit(-1);
    }
  }

  // Wait for all threads to exit.

  for (i=0; i< threadCount; i++) {
    Tcl_JoinThread(threadID[i], NULL);
  }

  // Use Tcl_Exit() instead of exit() to 
  //   ensure Tcl closes cleanly.
  Tcl_Exit(0);
}
