set defineGame {
  # Index is a location 
  # Value is a list of direction/new location pairs
  array set places {
    house {up roof down basement}
    roof {down house}
    basement {up house}
  }
  
  # Description to print for locations
  array set descript {
    house {You are inside a single room house}
    roof {You are on the roof.  Look at those clouds!}
    basement {You are in the basement.  It's spooky here.}
  }
  
  # Print the description and possible moves.
  proc describe {channel name index operation} {
    global places descript 
    upvar $name myState
  
    puts $channel "\n$descript($myState(location))"
  
    foreach {direction destination} $places($myState(location)) {
      puts $channel "You can go $direction (to the $destination)"
    }
    flush $channel
  }

  # Teleport to a destination
  proc teleport {destination player channel} {
    upvar #0 $player myState
    global places 
    if {[info exists places($destination)]} {
      set myState(location) $destination
    } else {
      puts $channel "You can't go to $destination"
    }
  }
  
  # Go in a direction
  proc go {direction player channel} {
    global places 
    upvar #0 $player myState
  
    array set moves $places($myState(location))
  
    if {![info exists moves($direction)]} {
      puts $channel "You can't go $direction from $myState(location)"
    } else {
      set myState(location) $moves($direction)
    }
  }
}

proc accptSock {channel ip port} {
  global sock
  global defineGame

  set sock(ch) $channel
  
  # Create a new save interpreter
  set ni [interp create -safe]
  
  # $id will hold the name of the player state
  # variable.  It will be used to hold state within
  # the slave interpreter.
  set id player_$ni

  # Transfer the channel to the slave.
  interp transfer {} $channel $ni

  # Send the variable and procedure definitions to
  # the new interpreter
  $ni eval $defineGame

  # Define a readData procedure in the new interp
  $ni eval [list proc readData $channel \
      "eval \[gets $channel\] $id $channel"]
  
  # Assign a fileevent to cause readData to be evaluated
  # when there is data to be read.
  $ni eval [list fileevent $channel readable \
      "readData $channel"]

  # put a trace on the player state variable's 
  # location index
  $ni eval [list trace add variable ${id}(location) \
      write "describe $channel"]

  # Initialize the player to the house 
  # The describe procedure will be invoked by trace
  # within the slave interpreter
  $ni eval [list set ${id}(location) house]
}

socket -server accptSock 12345

